from django import template

register = template.Library()


@register.filter
def get_for_model(d, row):
    return [o for o in d if o.content_type.model_class() == row['model_class']]

@register.filter
def get_item(d, key):
    return d.get(key, None)



"""
Form widget and field.

PermissionSelectMultipleField
    An optionnal form field purely for your own convenience.

PermissionSelectMultipleWidget
    The actual permissions widget.
"""
from string import lower
from django import forms
from django import template
from django.template.loader import get_template
from django.utils.safestring import mark_safe
from django.contrib.auth.models import Permission
import itertools

from .settings import EXCLUDE_APPS, EXCLUDE_MODELS


class PermissionSelectMultipleWidget(forms.CheckboxSelectMultiple):
    """
    Child of CheckboxSelectMultiple which renders
    `permissions_widget/widget.html` to display the form field.
    """
    def render(self, name, value, attrs=None, choices=()):
        if value is None:
            value = []

        table = []
        row = None
        last_app = None
        last_model = None
        permission_types = {}

        for permission in self.choices.queryset:
            codename = permission.codename
            model_part = "_" + permission.content_type.model
            permission_type = codename
            if permission_type.endswith(model_part):
                permission_type = permission_type[:-len(model_part)]
            app = permission.content_type.app_label

            setattr(permission, "permission_type", permission_type)
            setattr(permission, "value", permission.pk)

            model_class = permission.content_type.model_class()
            model_class_name = lower(model_class.__name__) if model_class else None
            model_verbose_name = model_class._meta.verbose_name if model_class else None

            if app in EXCLUDE_APPS:
                continue

            if model_class_name and u'%s.%s' % (app, model_class_name) in EXCLUDE_MODELS:
                continue

            permission_types.setdefault(permission_type, [])
            permission_types[permission_type].append(permission)

            is_app_or_model_different = last_model != model_class or last_app != app
            if is_app_or_model_different:
                row = dict(model=model_verbose_name, model_class=model_class, app=app, permissions={})

            # place permission
            row['permissions'][permission_type] = {
                'value': permission.pk,
                'name': permission.name,
            }

            if is_app_or_model_different:
                table.append(row)

            last_app = app
            last_model = model_class

        permission_types_standard = {k: v for k, v in permission_types.iteritems() if len(v) > 1}
        permission_types_custom = [p for p in itertools.chain(*[v[1] for v in permission_types.iteritems()]) if p.permission_type not in permission_types_standard]

        t = get_template('permissions_widget/widget.html')
        c = template.Context({
            'name': name,
            'value': value,
            'table': table,
            'permission_types': permission_types_standard,
            'permission_types_custom': permission_types_custom,
        })
        return mark_safe(t.render(c))


class PermissionSelectMultipleField(forms.ModelMultipleChoiceField):
    """
    Simple child of forms.ModelMultipleChoiceField which pre-sets
    queryset=Permission.objects.all(). It's an optionnal item here for your
    convenience.
    """
    widget = PermissionSelectMultipleWidget

    def __init__(self, queryset=None, *args, **kwargs):
        if queryset is None:
            queryset = Permission.objects.all()
        super(PermissionSelectMultipleField, self).__init__(queryset, *args,
                **kwargs)



"""
Settings for permissions_widget.

EXCLUDE_APPS
    The permissions widget will exclude any permission for any model in any app
    in the EXCLUDE_APPS list. It contains sensible defaults which you can
    override: sessions, admin and contenttypes for example, as in most cases
    users won't even have the possibility of adding/changing/deleting sessions,
    logentries and content types so why even bother proposing permissions for
    them ? This would just confuse the admin.
    Can be overridden in settings.PERMISSIONS_WIDGET_EXCLUDE_APPS.
EXCLUDE_MODELS
    The permissions widget will exclude any permission for any listed model.
    Models should be listed in the form of `app.model`.
    Can be overridden in settings.PERMISSIONS_WIDGET_EXCLUDE_MODELS.
PATCH_GROUPADMIN
    If True, `permissions_widget.admin` will override the registered GroupAdmin
    form's user_permission field to use this widget for permissions.
    Can be overridden (ie. to False) in
    settings.PERMISSIONS_WIDGET_PATCH_GROUPADMIN.
PATCH_USERADMIN
    If True, `permissions_widget.admin` will override the registered UserAdmin
    form's user_permission field to use this widget for permissions.
    Can be overridden (ie. to False) in
    settings.PERMISSIONS_WIDGET_PATCH_USERADMIN.
"""
from django.conf import settings


EXCLUDE_APPS = getattr(settings, 'PERMISSIONS_WIDGET_EXCLUDE_APPS', [
    'sites', 'reversion', 'contenttypes', 'admin', 'sessions',
    'easy_thumbnails',])
EXCLUDE_MODELS = getattr(settings, 'PERMISSIONS_WIDGET_EXCLUDE_MODELS', [
    'auth.permission',])
PATCH_USERADMIN = getattr(settings, 'PERMISSIONS_WIDGET_PATCH_USERADMIN', True)
PATCH_GROUPADMIN = getattr(settings, 'PERMISSIONS_WIDGET_PATCH_GROUPADMIN',
                           True)



"""
Patch User and/or Group admin form to use the permission widget depending on
settings.

The patch works like this:

- get the user or group model,
- get the registered ModelAdmin from `admin.site` for this model,
- get the form of this ModelAdmin,
- inherit from the form and replace the permissions field,
- inherit from the modeladmin and replace the form,
- unregister the model admin,
- register the new inheriting model admin using the inheriting form with the
  permissions field override.
"""
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from django.contrib import admin
from django import forms

from .forms import PermissionSelectMultipleField


User = get_user_model()
OriginalUserAdmin = admin.site._registry[User].__class__
OriginalUserChangeForm = OriginalUserAdmin.form

class NewUserChangeForm(OriginalUserChangeForm):
    user_permissions = PermissionSelectMultipleField(required=False)


class NewUserAdmin(OriginalUserAdmin):
    form = NewUserChangeForm

admin.site.unregister(User)
admin.site.register(User, NewUserAdmin)


OriginalGroupAdmin = admin.site._registry[Group].__class__
OriginalGroupChangeForm = OriginalGroupAdmin.form


class NewGroupChangeForm(OriginalGroupChangeForm):
    permissions = PermissionSelectMultipleField(required=False)


class NewGroupAdmin(OriginalGroupAdmin):
    form = NewGroupChangeForm

admin.site.unregister(Group)
admin.site.register(Group, NewGroupAdmin)
